#include <stdint.h>
#include "hardware.h"
#include "msp430x20x2.h"

#define RED_LED   BIT0
#define GREEN_LED BIT6
#define LED_DIR   P1DIR
#define LED_OUT   P1OUT

#define BUTTON      BIT3
#define BUTTON_OUT  P1OUT
#define BUTTON_DIR  P1DIR
#define BUTTON_IN   P1IN
#define BUTTON_REN  P1REN

#define     TXD                   BIT1                      // TXD on P1.1
#define     RXD                   BIT2                      // RXD on P1.2

static bool g_wakeupFlag = false;
static uint16_t g_txByte = 0;
static uint8_t g_txBitCount = 0;

void sendByte(char byte)
{
  // neues Byte mit Start-Bit, Stop-Bit und ein wenig Pause in Auftrag geben
  g_txByte = (uint16_t(byte) << 1) | 0x600;
  g_txBitCount = 10;
  
  // CPU anhalten. Wenn es im Interrupt fertig gesendet wurde,
  // dann geht es weiter
  lpm0Halt();
}

#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A (void)
{
  // erstmal schauen, ob Bytes versendet werden mssen
  if (g_txBitCount) {
    if (g_txByte & 1) {
      P1OUT |= TXD;
    } else {
      P1OUT &= ~TXD;
    }
    g_txByte >>= 1;
    g_txBitCount--;
    if (g_txBitCount == 0) wakeup();
  }

  // User Interrupt Routine aufrufen
  onInterrupt();
  
  // wenn das Wakeup-Flag gesetzt wurde, dann Hauptprogramm
  // nach Interruptbeendigung weiterlaufen lassen
  if (g_wakeupFlag) {
    __bic_SR_register_on_exit(CPUOFF);
    g_wakeupFlag = false;
  }
}

void initHardware()
{
  // Watchdog ausschalten
  WDTCTL = WDTPW + WDTHOLD;
  
  // ADC auf Kanal 4 einstellen, mit der 1,5 V internen Referenz
  ADC10CTL1 = INCH_4 | ADC10DIV_3;
  ADC10CTL0 = SREF_1 | ADC10SHT_3 | REFON | ADC10ON | ADC10IE;
  ADC10AE0 = 1 << 4;

  // LED Pins auf Ausgang setzen
  LED_DIR |= RED_LED | GREEN_LED;
  
  // falls der Quarz nicht luft, oder zum ausprobieren der
  // richtigen CXCAP-Werte : ACLK auf die rote LED ausgeben
  //P1SEL |= RED_LED;
  
  // TXD auf Ausgang und initial auf 1 setzen, fr RS232 "Mark"
  P1OUT |= TXD;
  P1DIR |= TXD;
  
  // Button-Eingang konfigurieren
  BUTTON_DIR &= ~BUTTON;
  BUTTON_REN |= BUTTON;

  // rote LED an
  LED_OUT |= RED_LED;

  // den Low Frequency Mode fr den externen Quarzeingang setzen
  BCSCTL1 &= ~XTS;
  
  // durch 1 teilen
  BCSCTL1 |= DIVA_0;
 
  // 32768 Hz Modus einstellen
  BCSCTL3 &= ~(LFXT1S0 | LFXT1S1);
   
  // 12.5 pF einstellen
  BCSCTL3 |= XCAP0 | XCAP1;

  // zunchst ggf. aktiven Capture/Compare Timer-Interrupt ausschalten  
  TACCTL0 =  0;

  // Timer im "Up" Modus laufen lassen, mit ACLK getaktet
  TACTL = TASSEL_1 | MC_1 | ID_0;
  
  // Timer stoppen
  // (siehe Kapitel 12.2.2 vom MSP430x2xx Family User's Guide)
  TACCR0 = 0;
  
  // neuen Grenzwert festlegen fr ca. 2340,57 Hz
  TACCR0 = 13;
  
  // Capture/Compare Interrupt einschalten
  TACCTL0 = CCIE;
}

void lpm0Halt()
{
  // Low Power Mode 0 aktivieren und Interrupts aktivieren
  // Die CPU hlt hiernach an und luft nur noch im Interrupt
  __bis_SR_register(LPM0_bits | GIE);
}

void wakeup()
{
  g_wakeupFlag = true;
}

uint16_t getAdc()
{
    ADC10CTL0 |= ENC | ADC10SC;             // Sampling and conversion start
    lpm0Halt();
    return ADC10MEM;
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
  __bic_SR_register_on_exit(CPUOFF);        // Return to active mode
}

void setGreenLed(bool on)
{
  if (on) {
    LED_OUT |= GREEN_LED;
  } else {
    LED_OUT &= ~GREEN_LED;
  }
}
